#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>

using namespace std;

#include <BsplineEval/bsplineval.h>

#include "config.h"
#include "main.h"


static void get_delta(
  spline_coef& scoef,
  int num_theta,
  int num_z, 
  double* init_t,
  double* init_z,
  double* delta_theta, 
  double* delta_z 
) {

  *init_t = scoef.tmin;
  *init_z = scoef.zmin;

  if ( 0 == num_theta-1 )
    *delta_theta = 0.0;
  else
    *delta_theta = (scoef.tmax - scoef.tmin) / (num_theta-1);

  if ( 0 == num_z - 1 )
    *delta_z = 0.0;
  else
    *delta_z = (scoef.zmax - scoef.zmin ) / (num_z-1);
}

/*
 *  Make a grid of dimension max_num by max_num of the points inside the
 *  interior knots of the splines to check the robustness of the routines.
 */
void
eval_splines(
  SplineInfo& spinfo,
  int max_num,
  string label,
  string outfile,
  EvalSpline      func_eval,
  EvalSplineDeriv func_eval_derivs
  ) {

  int i, j;

  BsplineErr bsperr;

  double init_t, delta_t, init_z, delta_z;

  get_delta( spinfo.scoef, max_num, max_num, &init_t, &init_z, 
	     &delta_t, &delta_z );

  ofstream out( outfile.c_str() );

  out.setf(ios_base::scientific, ios_base::floatfield);

  for ( i = 0; i < max_num; i++ ) 
  {
    double tt = init_t + i * delta_t;

    for ( j = 0; j < max_num; j++ ) 
    {

      BsplineResult bspres;
      char buf[1024];
      char buffmt[1024] = "%.14f\t%.14f\t%.14f\t%.14f\t%.14f";

      double result = 0;
      double zz = init_z + j * delta_z;

      bsperr = func_eval( tt, zz, spinfo, &result );

      if ( Bspline_Success != bsperr ) 
      {
	cerr << label << "eval error @ t=" << tt << ", z= " << zz << endl;
	Bspline_err_msg( bsperr, stderr );
	exit(EXIT_FAILURE);
      }

      bsperr = func_eval_derivs( tt, zz, spinfo, &bspres );
      if ( Bspline_Success != bsperr )
      {
	cerr << label << " deriv error @ t=" << tt << ", z= " << zz << endl;
	Bspline_err_msg( bsperr, stderr );
	exit(EXIT_FAILURE);
      }

      sprintf( buf, buffmt, tt, zz, result, bspres.dsdz, bspres.dsdt );
      out << buf << endl;
    }
  }
}
